﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PV178_WinForms_Demo
{
    public partial class AsyncOperationsForm : Form
    {
        object exclusiveUIAccess = new object();

        public AsyncOperationsForm()
        {
            InitializeComponent();
        }

        private void btnSync_Click(object sender, EventArgs e)
        {
            ReportProgress(0);
            Thread.Sleep(1000);
            ReportProgress(0.25f);
            Thread.Sleep(1000);
            ReportProgress(0.5f);
            Thread.Sleep(1000);
            ReportProgress(0.75f);
            Thread.Sleep(1000);
            ReportProgress(1f);
            MessageBox.Show("Done");
        }

        private async void btnAwait_Click(object sender, EventArgs e)
        {
            ReportProgress(0);
            await Task.Delay(1000);
            ReportProgress(0.25f);
            await Task.Delay(1000);
            ReportProgress(0.5f);
            await Task.Delay(1000);
            ReportProgress(0.75f);
            await Task.Delay(1000);
            ReportProgress(1f);
            MessageBox.Show("Done");
        }

        private async void btnCAwait_Click(object sender, EventArgs e)
        {
            ReportProgress(0);
            await Task.Delay(1000).ConfigureAwait(false);
            ReportProgress(0.25f);
        }

        private void btnBgWorker_Click(object sender, EventArgs e)
        {
            backgroundWorker.RunWorkerAsync();
        }

        private void backgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            backgroundWorker.ReportProgress(0);
            Thread.Sleep(1000);
            backgroundWorker.ReportProgress(25);
            Thread.Sleep(1000);
            backgroundWorker.ReportProgress(50);
            Thread.Sleep(1000);
            backgroundWorker.ReportProgress(75);
            Thread.Sleep(1000);
            backgroundWorker.ReportProgress(100);
            MessageBox.Show("Done");
        }

        private void backgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            ReportProgress(e.ProgressPercentage / 100f);
        }

        private void btnThread_Click(object sender, EventArgs e)
        {
            Task.Run(() =>
            {
                ReportProgressThreadSafe(0);
                Thread.Sleep(1000);
                ReportProgressThreadSafe(0.25f);
                Thread.Sleep(1000);
                ReportProgressThreadSafe(0.5f);
                Thread.Sleep(1000);
                ReportProgressThreadSafe(0.75f);
                Thread.Sleep(1000);
                ReportProgressThreadSafe(1f);
                MessageBox.Show("Done");
            });
        }

        private void btnDeadlock_Click(object sender, EventArgs e)
        {
            var task = Task.Run(() => ReportProgressThreadSafeDeadlockable(1f));
            task.Wait();
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            ReportProgress(0f);
        }

        void ReportProgress(float progress)
        {
            var steps = (int)Math.Round(progress * progressBar.Maximum);
            progressBar.Value = steps;
        }

        void ReportProgressThreadSafe(float progress)
        {
            var steps = (int)Math.Round(progress * progressBar.Maximum);
            progressBar.BeginInvoke(() => progressBar.Value = steps);
        }

        void ReportProgressThreadSafeDeadlockable(float progress)
        {
            var steps = (int)Math.Round(progress * progressBar.Maximum);
            progressBar.Invoke(() => progressBar.Value = steps);
        }
    }
}
