﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using SixLabors.ImageSharp;

namespace PV178_WinForms_Demo
{
    public partial class DemoPhotoFilterForm : Form
    {
        private const string FileDialogFilterString = "PNG Image (*.png)|*.png|JPEG Image (*.jpg;*jpeg)|(*.jpg;*jpeg)|BMP Image (*.bmp)|*.bmp";
        private PictureBox OriginalPictureBox => null;
        private PictureBox PreviewPictureBox => null;
        private ComboBox PhotoFilterComboBox => null;

        private Image _originalImage;

        private Image OriginalImage
        {
            get => _originalImage;
            set
            {
                OriginalPictureBox.Image?.Dispose();
                _originalImage?.Dispose();

                _originalImage = value;
                OriginalPictureBox.Image = value?.ToSystemDrawingImage();

                EditedImage = Filter.Apply(value, Strength);
            }
        }

        private Image _editedImage;

        private Image EditedImage
        {
            get => _editedImage;
            set
            {
                PreviewPictureBox.Image?.Dispose();
                _editedImage?.Dispose();

                _editedImage = value;
                PreviewPictureBox.Image = value.ToSystemDrawingImage();
            }
        }

        private IImageFilter Filter => PhotoFilterComboBox.SelectedItem as IImageFilter;

        private IImageFilter[] AvailableFilters { get; } = typeof(Program).Assembly.GetTypes()
            .Where(x =>
                !x.IsAbstract &&
                typeof(IImageFilter).IsAssignableFrom(x) &&
                x.GetConstructor(Type.EmptyTypes) is not null)
            .Select(x => Activator.CreateInstance(x) as IImageFilter)
            .OrderBy(x => x.Name)
            .ToArray();

        private float _strength;

        private float Strength
        {
            get => _strength;
            set
            {
                _strength = value;
                EditedImage = Filter.Apply(OriginalImage, value);
            }
        }

        public DemoPhotoFilterForm()
        {
            InitializeComponent();
            RegisterMenuItemEvents();
        }

        private void RegisterMenuItemEvents()
        {
            // TODO
        }
    }
}